#!/bin/sh
#
#  avast! VPS update script
#  Put a call to this in your root crontab
#
#  Copyright(c) 2004-2006	ALWIL Software
#
#  This script will use 'wget' and 'md5sum' to check whether the 400.vps
#  database file is up to date or should be downloaded and installed using
#  'avastvpsreload'. It's shell-based equivalent of the avastvpsupdate.pl
#  script. In the case of missing 'wget', 'lynx' is automatically tried.
#  Also, 'md5' as an "md5sum" alternative will be tried, both versions.
#
#  The same task can be performed by avastvpsupdate.pl, but there are
#  other dependencies on Perl (and its modules MD5 and URI).
#
#  Where to download mentioned utilities:
#  md5sum sources: http://archiv.tu-chemnitz.de/pub/1999/0004/data/md5 
#  md5 sources: http://www.fourmilab.ch/md5/md5.tar.gz
#  wget sources: http://ftp.gnu.org/gnu/wget/wget-1.10.2.tar.gz
#  lynx sources: http://lynx.isc.org/release/lynx2.8.5.tar.gz
#
#  Binaries for Solaris: http://www.sunfreeware.com
#  note: for wget, modify proxy setting in /usr/local/etc/wgetrc
#  note: for lynx, place link to gzip to /usr/local/bin

##### global defaults

VUS_EXE_SEARCH_PATH="/usr/local/bin:/usr/local/sbin:/bin:/sbin:/usr/bin:/usr/sbin"
VUS_CONFIG_PATHS="/etc /usr/local/etc"
VUS_UPDATE_CONFIG_FILE="avastvpsupdate.conf"
VUS_ENGINE_CONFIG_FILE="avastengine.conf"
VUS_UPDATE_NEW_CONFIG_FILE="avastvpsupdaterc"
VUS_ENGINE_NEW_CONFIG_FILE="avastenginerc"
VUS_VPS_FILE="400.vps"
VUS_MD5_FILE="400vps.md5"
VUS_SCRIPT_LOGFILE="`basename $0`.log"

# default values when everything else fails
VUS_DEFAULT_VPS_LOCATION="http://files.avast.com/files/latest"
VUS_DEFAULT_TEMP_DIRECTORY="/var/tmp"
VUS_DEFAULT_AVAST_DATADIR="/var/run/avast4"
VUS_TEMPORARY_UNIQUE_PREFIX=`date +%h-%m-%y-%H-%M-%S`-$$

# some Bulgarian constants from the previous perl version
VUS_HARDWIRED_USE_PROXY=2
VUS_MD5_TEST_STRING=AaZz09
VUS_MD5_TEST_RESULT=13b05a709b228cb212e4dfc06050caa3

##### functions

#args: file

vus_do_md5()
    {
    if [ -f "$1" ]
	then
	    vus_do_md5_result=`$vus_md5sum $vus_md5sum_options $1 2>/dev/null`
	    if [ "$vus_do_md5_result" ]
		then
		    if [ `echo $vus_do_md5_result | wc -w` -gt 2 ]
			then
			    echo `echo $vus_do_md5_result | cut -d= -f2`
			else
			    echo `echo $vus_do_md5_result | cut -d\  -f1`
			fi
		else
		    echo FAILED
		fi
	else
	    echo NONE
	fi
    }

#args: executable
vus_which()
    {
    vus_temp_value=1
    vus_temp_target=x
    while [ "$vus_temp_target" ]
	do
	    vus_temp_target=`echo $PATH | cut -d: -f$vus_temp_value`
	    if [ "$vus_temp_target" -a -x "$vus_temp_target/$1" ]
		then
		    echo "$vus_temp_target/$1"
		    return 0
		fi
	    vus_temp_value=`expr $vus_temp_value + 1`
	done
    }

#args: error_code, error_string
vus_error_and_exit()
    {
    if [ $vus_var_logdir != 0 ]
	then
	    echo `date` ERROR-$1: $2. >>$vus_var_logdir/$VUS_SCRIPT_LOGFILE
	fi
    echo
    echo ERROR-$1: $2.
    exit $1
    }

#args: target_variable, located_file_alternative, located_file_default, dir_path ...
vus_locate_config_file()
    {
    vus_temp_target=$1
    vus_temp_value=$3
    
    if [ -f $HOME/.$2 ]
		then
		    eval $vus_temp_target=$HOME/.$2
		    return 0
		fi
		
    if [ -f $HOME/.$3 ]
		then
		    eval $vus_temp_target=$HOME/.$3
		    return 0
		fi
		
    shift 3
    for vus_temp_line in $*
	do
	    if [ -f ${vus_temp_line}/$vus_temp_value ]
		then
		    eval $vus_temp_target=${vus_temp_line}/$vus_temp_value
		    return 0
		fi
	done
    vus_error_and_exit 1 "Cannot locate configuration file $vus_temp_value"
    }

#args: target_variable, configuration_file, configuration_token, default
vus_set_configuration_variable()
    {
    vus_temp_line=`grep "^${3}=" $2 2>/dev/null`
    if [ "$vus_temp_line" ] 
	then 
	    vus_temp_value=`echo $vus_temp_line | cut -d= -f2 | cut -f1`
	    eval $1=$vus_temp_value
	else
	    eval $1=$4
	fi
    }

##### main commands

# extend the path and temporarily disable logging
vus_var_logdir=0
PATH=$PATH:$VUS_EXE_SEARCH_PATH
export PATH

# locate configuration files
vus_locate_config_file vus_conf_engine $VUS_ENGINE_NEW_CONFIG_FILE $VUS_ENGINE_CONFIG_FILE $VUS_CONFIG_PATHS
vus_locate_config_file vus_conf_update $VUS_UPDATE_NEW_CONFIG_FILE $VUS_UPDATE_CONFIG_FILE $VUS_CONFIG_PATHS

# enable logging and check whether to update
vus_set_configuration_variable vus_var_logdir $vus_conf_engine "Avast4LogDir" 0
vus_set_configuration_variable vus_var_update $vus_conf_update "Update" 1
if [ $vus_var_update -ne 0 ]
    then

# check the presence of necessary utilities
	if [ x`vus_which md5` != x ]
	    then
		vus_md5sum=md5
	    else
		vus_md5sum=md5sum
	    fi
	    
	if [ \( x`vus_which wget` = x -a x`vus_which lynx` = x \) -o x`vus_which avastvpsreload` = x -o x`vus_which $vus_md5sum` = x ]
	    then
		vus_error_and_exit 2 "Utilities $vus_md5sum, avastvpsreload and wget (or lynx) not available, see $0"
	    else

# get configuration variables or defaults for the engine, test them
	vus_set_configuration_variable vus_var_tempdir $vus_conf_engine "Avast4TempDir" $VUS_DEFAULT_TEMP_DIRECTORY
	vus_var_target=$vus_var_tempdir/$VUS_TEMPORARY_UNIQUE_PREFIX.$VUS_MD5_FILE
	echo $VUS_MD5_TEST_STRING >$vus_var_target 2>/dev/null
	if [ $? -ne 0 ]
	    then
		vus_error_and_exit 3 "Cannot write to temporary directory '$vus_var_tempdir'"
	    fi

# test the md5sum behavior and try to use the proper switches if necessary
	vus_md5sum_options=""
	if [ `vus_do_md5 $vus_var_target` != $VUS_MD5_TEST_RESULT ]
	    then
		vus_md5sum_options="$vus_md5sum_options -l"
		if [ `vus_do_md5 $vus_var_target` != $VUS_MD5_TEST_RESULT ]
		    then
			vus_error_and_exit 7 "The md5sum utility '$vus_md5sum' seems to be broken"
		    fi
	    fi

# get configuration variables or defaults for downloading
	vus_set_configuration_variable vus_var_vpslocation $vus_conf_update "VpsLocation" $VUS_DEFAULT_VPS_LOCATION
	vus_set_configuration_variable vus_var_netacc $vus_conf_update "NetAcc" 0
	if [ $vus_var_netacc -eq $VUS_HARDWIRED_USE_PROXY ]
	    then
		vus_set_configuration_variable vus_var_netip $vus_conf_update "NetIP" $VUS_DEFAULT_PROXY_IP
		vus_set_configuration_variable vus_var_netport $vus_conf_update "NetPort" $VUS_DEFAULT_PROXY_PORT
		vus_set_configuration_variable vus_var_netuser $vus_conf_update "NetUser" ""
		vus_set_configuration_variable vus_var_netpwd $vus_conf_update "NetPwd" ""
	    fi

# prepare wget or lynx to be used for downloading
	if [ x`vus_which wget` = x ]
	    then
		vus_var_downloader="lynx"
		if [ $vus_var_netacc -eq $VUS_HARDWIRED_USE_PROXY ]
		    then
			export http_proxy=$vus_var_netip:$vus_var_netport
		    fi
		if [ x$vus_var_netuser != x ]
		    then
			vus_var_downloader="$vus_var_downloader -pauth=$vus_var_netuser:$vus_var_netpwd"
		    fi
		vus_var_downloader="$vus_var_downloader -source -reload"
		vus_var_output=""
		vus_var_stdout="\$vus_var_target"
	    else
		vus_var_downloader="wget"
		if [ $vus_var_netacc -eq $VUS_HARDWIRED_USE_PROXY ]
		    then
			export http_proxy=$vus_var_netip:$vus_var_netport
		    fi
		if [ x$vus_var_netuser != x ]
		    then
			vus_var_downloader="$vus_var_downloader --proxy-user=$vus_var_netuser"
		    fi
		if [ x$vus_var_netpwd != x ]
		    then
			vus_var_downloader="$vus_var_downloader --proxy-passwd=$vus_var_netpwd"
		    fi
		vus_var_downloader="$vus_var_downloader -q -r"
		vus_var_output="-O \$vus_var_target"
		vus_var_stdout="/dev/null"
	    fi

# download the md5 of the latest database from the web
	    $vus_var_downloader $vus_var_vpslocation/$VUS_MD5_FILE `eval echo $vus_var_output` >`eval echo $vus_var_stdout` 2>/dev/null
	    if [ $? -ne 0 ]
	    	then
		    vus_error_and_exit 4 "Cannot download $VUS_MD5_FILE from $vus_var_vpslocation"
		fi
	    vus_var_md5_d=`cat $vus_var_tempdir/$VUS_TEMPORARY_UNIQUE_PREFIX.$VUS_MD5_FILE | cut -d\  -f1`
	    rm -f $vus_var_target

# compare md5 of the current database with the downloaded md5
	    vus_set_configuration_variable vus_var_datadir $vus_conf_engine "Avast4DataDir" $VUS_DEFAULT_AVAST_DATADIR
	    vus_var_md5_o=`vus_do_md5 $vus_var_datadir/$VUS_VPS_FILE`
	    if [ $vus_var_md5_o = $vus_var_md5_d ]
		then
		    exit 0	#we don't log all new-vps-polls
		fi

# download and verify the new vps
	    vus_var_target=$vus_var_tempdir/$VUS_TEMPORARY_UNIQUE_PREFIX.$VUS_VPS_FILE
	    $vus_var_downloader $vus_var_vpslocation/$VUS_VPS_FILE `eval echo $vus_var_output` >`eval echo $vus_var_stdout` 2>/dev/null
	    if [ $? -ne 0 ]
		then
		    vus_error_and_exit 5 "Cannot download $VUS_VPS_FILE from $vus_var_vpslocation"
		fi
	    vus_var_md5_o=`vus_do_md5 $vus_var_target`
	    if [ $vus_var_md5_o != $vus_var_md5_d ]
		then
		    rm -f $vus_var_tempdir/$VUS_TEMPORARY_UNIQUE_PREFIX.$VUS_VPS_FILE
		    vus_error_and_exit 6 "The MD5 of downloaded $VUS_VPS_FILE differs from $vus_var_vpslocation/$VUS_MD5_FILE"
		fi

# install and reload the new vps with regards to the uid
	    mv $vus_var_tempdir/$VUS_TEMPORARY_UNIQUE_PREFIX.$VUS_VPS_FILE $vus_var_tempdir/$VUS_VPS_FILE
	    vus_var_uid=`id | cut -d= -f2 | cut -d\( -f1`
	    vus_var_reloader="avastvpsreload -d $vus_var_tempdir -l $vus_var_datadir"
	    if [ $vus_var_uid -eq 0 ]
		then
		    vus_var_reloader="$vus_var_reloader -a"
		fi
	    $vus_var_reloader >/dev/null 2>&1				#error codes might be various, we'll verify the succes below
	    vus_var_vps=`avastvpsreload -V | grep VPS`
	    if echo "$vus_var_vps" | grep "?" -s >/dev/null 2>&1
		then
		    vus_error_and_exit 6 "Reloading of the downloaded VPS file $vus_var_datadir/$VUS_VPS_FILE failed"
		else
		
# the update was succesfull, so log the final message and exit

	vus_error_and_exit 0 "New $vus_var_vps succesfully installed"
	
    fi
    fi
    fi
